Package: MSnbase
Title: Base Functions and Classes for Mass Spectrometry and Proteomics
Version: 2.39.4
Description: MSnbase provides infrastructure for manipulation,
             processing and visualisation of mass spectrometry and
             proteomics data, ranging from raw to quantitative and
             annotated data.
Authors@R: c(person(given = "Laurent", family = "Gatto",
                    email = "laurent.gatto@uclouvain.be",
                    comment = c(ORCID = "0000-0002-1520-2268"),
                    role = c("aut","cre")),
             person(given = "Guangchuang", family = "Yu",
                    email = "guangchuangyu@gmail.com",
                    role = "ctb"),
             person(given = "Samuel", family = "Wieczorek",
                    email = "samuel.wieczorek@cea.fr",
                    role = "ctb"),
             person(given = "Vasile-Cosmin", family = "Lazar",
                    email = "v.cosmin.lazar@googlemail.com",
                    role = "ctb"),
             person(given = "Vladislav", family = "Petyuk",
                    email = "petyuk@gmail.com",
                    role = "ctb"),
             person(given = "Thomas", family = "Naake",
                    email = "tn299@cam.ac.uk",
                    role = "ctb"),
             person(given = "Richie", family = "Cotton",
                    email = "richierocks@gmail.com",
                    role = "ctb"),
             person(given = "Steffen", family = "Neumann",
                    email = "sneumann@ipb-halle.de",
                    role = "ctb"),
             person(given = "Arne", family = "Smits",
                    email = "arne.smits@embl.de",
                    role = "ctb"),
             person(given = "Martina", family = "Fischer",
                    email = "FischerM@rki.de",
                    role = "ctb"),
             person(given = "Adriaan", family = "Sticker",
                    email = "adriaan.sticker@ugent.be",
                    role = "ctb"),
             person(given = "Ludger", family = "Goeminne",
                    email = " ludger.goeminne@vib-ugent.be",
                    role = "ctb"),
             person(given = "Lieven", family = "Clement",
                    email = "Lieven.Clement@ugent.be",
                    role = "ctb"),
             person(given = "Sigurdur", family = "Smarason",
                    email = " Sigurdur.Smarason@eurac.edu",
                    role = "ctb"),
             person(given = "Miguel", family = "Cosenza-Contreras",
                    email = "migueljcc5@gmail.com",
                    role = "ctb"),
             person(given = "Pascal", family = "Maas",
                    email = "p.maas@lacdr.leidenuniv.nl",
                    comment = c(ORCID = "0000-0001-9379-6041"),
                    role = "ctb"),
             person(given = "Johannes", family = "Rainer",
                    email = "Johannes.Rainer@eurac.edu",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-6977-7147")),
             person(given = "Sebastian", family = "Gibb",
                    email = "mail@sebastiangibb.de",
                    role = c("aut", "cre"),
                    comment=c(ORCID="0000-0001-7406-4443")))
Author: Laurent Gatto, Johannes Rainer and Sebastian Gibb with
    contributions from Guangchuang Yu, Samuel Wieczorek, Vasile-Cosmin
    Lazar, Vladislav Petyuk, Thomas Naake, Richie Cotton, Arne Smits,
    Martina Fisher, Ludger Goeminne, Adriaan Sticker, Lieven
    Clement and Pascal Maas.
Maintainer: Laurent Gatto <laurent.gatto@uclouvain.be>
Depends: R (>= 3.5), methods, BiocGenerics (>= 0.7.1), Biobase (>=
        2.15.2), mzR (>= 2.29.3), S4Vectors, ProtGenerics (>= 1.29.1)
Imports: MsCoreUtils, PSMatch (>= 1.15.3), PTMods (>= 0.99.5),
        BiocParallel, IRanges (>= 2.13.28), plyr, vsn, grid, stats4,
        affy, impute, pcaMethods, MALDIquant (>= 1.16), mzID (>=
        1.5.2), digest, lattice, ggplot2, scales, MASS, Rcpp
Suggests: testthat, gridExtra, microbenchmark, zoo, knitr (>= 1.1.0),
        Rdisop, pRoloc, pRolocdata (>= 1.43.3), magick, MsDataHub,
        msdata, roxygen2, rgl, rpx, AnnotationHub, BiocStyle (>=
        2.5.19), rmarkdown, imputeLCMD, norm, gplots, XML, shiny,
        magrittr, SummarizedExperiment, Spectra
LinkingTo: Rcpp
License: Artistic-2.0
LazyData: yes
VignetteBuilder: knitr
Encoding: UTF-8
BugReports: https://github.com/lgatto/MSnbase/issues
URL: https://lgatto.github.io/MSnbase
biocViews: ImmunoOncology, Infrastructure, Proteomics,
        MassSpectrometry, QualityControl, DataImport
Roxygen: list(markdown=TRUE)
Collate: 'AllClassUnions.R' 'AllGenerics.R' 'DataClasses.R' 'MzTab.R'
        'NTR.R' 'RcppExports.R' 'TMT10.R' 'TMT11.R' 'TMT16.R' 'TMT6.R'
        'TMT7.R' 'averageMSnSet.R' 'cache.R' 'coerce.R'
        'combineFeatures.R' 'compfnames.R' 'environment.R'
        'fData-utils.R' 'fdata-selection.R' 'foi.R'
        'functions-Chromatogram.R' 'functions-MChromatograms.R'
        'functions-MIAPE.R' 'functions-MSnExp.R'
        'functions-MSnProcess.R' 'functions-MSnSet.R'
        'functions-MSpectra.R' 'functions-OnDiskMSnExp.R'
        'functions-ReporterIons.R' 'functions-Spectrum.R'
        'functions-Spectrum1.R' 'functions-Spectrum2.R'
        'functions-addIdentificationData.R' 'functions-mzR.R'
        'functions-plotting.R' 'hmap.R' 'iPQF.R' 'iTRAQ4.R' 'iTRAQ5.R'
        'iTRAQ8.R' 'iTRAQ9.R' 'imputation.R' 'map.R' 'matching.R'
        'methods-Chromatogram.R' 'methods-MChromatograms.R'
        'methods-MIAPE.R' 'methods-MSnExp.R' 'methods-MSnProcess.R'
        'methods-MSnSet.R' 'methods-MSnSetList.R' 'methods-MSpectra.R'
        'methods-OnDiskMSnExp.R' 'methods-ReporterIons.R'
        'methods-Spectrum.R' 'methods-Spectrum1.R'
        'methods-Spectrum2.R' 'methods-all.equal.R' 'methods-filters.R'
        'methods-mzR.R' 'methods-other.R' 'methods-pSet.R'
        'methods-updateObjectTo.R' 'methods-write.R' 'missing-data.R'
        'nadata.R' 'options.R' 'plotting-MSnExp.R' 'plotting-MSnSet.R'
        'plotting-Spectrum.R' 'plotting-Spectrum1.R'
        'plotting-Spectrum2.R' 'plotting-dataframe.R'
        'quantitation-MS2-isobaric.R' 'quantitation-MS2-labelfree.R'
        'readChromData.R' 'readMSData.R' 'readMSData2.R' 'readMSnSet.R'
        'readMzXMLData.R' 'readWriteMgfData.R' 'readWriteMzTab.R'
        'utils.R' 'writeMSData.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/MSnbase
git_branch: devel
git_last_commit: a69e0cb
git_last_commit_date: 2026-07-14
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
NeedsCompilation: yes
Packaged: 2026-07-14 21:54:58 UTC; biocbuild
Built: R 4.6.1; x86_64-apple-darwin20; 2026-07-15 05:39:50 UTC; unix
